from .all_courses import AllCoursesQueryData
from .canvas_course_data import Term, Assignment, AssignmentConnection, ModuleItemInterface, File, Page, \
    ModuleItem, \
    Module, ModuleConnection, CanvasCourseData as Course
