# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.6.1
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x0c\xda\
\x00\
\x00:\xf7x\xda\xdd[\xeb\x8f\xdb6\x12\xff\xbe\x7f\x85\
\xce\xc1\xa1\x09\x22\xc9\x22\xa9\xa7\xf7Q4\x09z\x09p\
\x87\x03.i\x0f\xe8\x97B\x96h[]=\x0cI^\
\xdb9\xdc\xff~3\xd4[\xa6\xed\x8d\xbb\xdd\x5c\xbaF\
di8\x1c\x0e\x7f3C\xceP\xce\xcd\xf7\xbb$V\
\x1ex^DYz;!\xba1Qx\x1ada\x94\
.o'?}\xfaQs'JQ\xfai\xe8\xc7Y\
\xcao'i6\xf9\xfe\xee\xea\xe6/\x9a\xa6\xbc\xcd\xb9\
_\xf2P\xd9F\xe5J\xf9\x90\xde\x17\x81\xbf\xe6\xca\xcb\
UY\xaeg\xd3\xe9v\xbb\xd5\xa3\x9a\xa8g\xf9r\xfa\
J\xd1\xb4\xbb\xab\xab\x9b\xe2ay\xa5(J\xd36\xe3\
\xbbu\x96\x97\xda>\x5cG\xb7\x13\xcf\xd0\x0d\xf17\x91\
\xf1\xec$<\xdb(,W\xb7\x13j\xd9\xe2q\xc5\xa3\
\xe5\xaa\xec\x9e\xa3\xf0v\x02#\x12\xc2j\xfe\x02\xe6\xb6\
\x86\x7f\xb3v\xd2\x86\xce\xe8p\xb8\x1e\x1eL\xa7\xcaK\
2\x137\xaf\xa9A\x19!\xd4\xa2\xc42^\x1b\x1e\xe1\
\xd4\xe0\x0bc\xf1j(8\xcc\x82\xd4O\x00\xab(\xc8\
R\x1d\x06\x1f\x0a\xcf6\xe5zS\xfe\xcaw%O\xab\
Q\x00\x9d\x1eT\xa2\x19\xbb\xb54\xd1\x7f`#1^\
\xb9\x8fa\x900*\xd6\xb1\xbf\x9fEi\x1c\xa5\xfc\x9a\
\xa7\xfe<\xe6\xda\xdc\x0f\xee\x97y\xb6I\xc3Y\xca\xb7\
R0\x17Q\xcc+=\xa7\xab,\xe1\xd3\xc2O\xa6I\
v\xefk)\xa8\xa6\x95<\x01\xb1%\xd7\xd7i\xa5?\
\xf8\xc9\xacX\xfb\x01\xf0\xafs^\xf0\xfc\x817\xf4\xb4\
\x985\xc2o'\xc7\xcc\x8fC\x89\xfe\xc5t0\xaf\xaa\
\x7f\x03^\xdb\xbf!\xe8E\xb6\xc9\x03\xbe\x00\x11\x5cO\
y9}\xf7\xe9]\xdb\xa8\x19zX\x86=1;\xc0\
\xe0~\xa0\xc3\x96\x89\xd1\x89\xe7yS\xd1\xdaq\xcb\xf8\
(\xb8\xd5\xb41Y\xad\xda\xc3\xf2\x91\x9cy\xb88:\
\xb6A\xa7\x94j\xc0\xa1\x15\xfb\xb4\xf4wZZ\xbc\xe8\
u\x0d\x82\xb6g\x80a\x15=\xf0 K\x92,-*\
\xf0\x06\xcca\xc7\xbc\xde\xe4\xb1\xe0\x08\x83)\x8fy\xc2\
\xd3\xb2\x98\x12\x9dL'w7\xadC\x22\xf4\xe1C\xc4\
\xb7(\x02\x1d'\xcf\xee\xc1R/\xf8\x82z\xd4\x9bT\
Tp\x87\x18h\x0b\xcb\xf1\xeaH\xa9\x82g\xee\x17\xbc\
~\x5c\xfbK\xd0*\xcer\xe0s\xc4_\xdd0\xcf\xf2\
\x90\xe7M\x93-\xfe\x06M\x19\x18>*\xf7\x18k\xd4\
2=\x83xM{\xeb\x95(\xbcc\x93\xb5\x16+?\
\xcc\xb6\x10\xda\xe3\xc6\xcfY\x96H\xc8\xc1\x0e\x88\xa6\xad\
;\xd6A\x0b\x0c\x01q,k\xd9\xe49`\xa8AD\
q\x98\x8b\xf8jfY\xac\xb2\xed2GL\x16~\xdc\
\x82\xd2v\xc5&m>\xcf`\xd42\xdf\x1c4\xc3\xaa\
\xb0A\xf3h\x9b4*\xc1\xf7\xd6\xbb1\x07\xca\xef\xcf\
S:\xca6J\xa1Q\xabW=\xe2Q\xe3\x08G\xb3\
\x10\x12\x83\xb8GXv'\x05\xf4\xccP\x8ff\x1aF\
\xabu#\x9e\xf5h\xddLR\x7f\xad\xa5Y\xc8\x0b9\
\x16\xa2\xfd\x00*\x04\xb0\xccb\x9e\xfb)\xae2\xa4]\
\xe2\xfb\x8b\xe7\xfc7\x1e\x94ge\xa3\xa8a\xbb0\xde\
&:\xdaM\xb4\x9d0_\x0dJ\xe2\xef\xa2$\xfa\xcc\
\xc1\x09\xc8\x98\x05;\x9f\xd2L\xb4\xaf\xfdru\x0e\x15\
\x8d\x87K\xae%Q\xb8\xce\xa2\xb4<\xcb}\x86QD\
\xed\x8a\x07\xf7<\x9fg~\x1eJ\xa2\x0bT\xbeo\xa2\
7$\xf8\x81\xe5c\xe0\xd8U\x07\x80\x11#4]b\
\xfc\xb4fo\xa9\xbb!\x15#\x05\xfbZ\xae\xcbZb\
\xb9\xc7Mb\xb7\xc7\x86\x96XmZ\xe1@\x7f\xd8\xf1\
\xa2\x22\x9a\xe3\x16\xd7\x0f\x04`N\xd6\xf5x\xe0\x91\x9d\
\x0a\x00G\xdd\x01E\xe3^Xdi\xbc\x1f\xc9\x0c\xb3\
\xb2\xe4\xe1Xd3\xf7^Z\x81\x7f\xbdU\x8b\xe0\xb2\
\xd5W\xa1\xe9B\x0d\xc6\x17\xac\xdf\xd2\xeb\xc5l\xd31\
`\xfa\x9d\xc4<ZF)\xe0\xa4\x11{D\x84\x0e\x1a\
\xed8\xbbUB\x99\x1e\xb1\x85\x14\xca\x06tB,\x8f\
J1s\xcf\x02\xdcXcH\xee\x8cl\xe8\xd6\xa1\xf1\
\xf7cz\x83\xd0b\xf1\xc5\xa0\x8e\xba\x0cA\xa5\x96\xe1\
\xb9\x06\xf3.5\xfd\x80x\xa9A\xa6\x87\x9b\xeb\xddM\
\xc8\x17E\xb7q\xe2\x13\x838B}\xfc\xfco\xb9\x1f\
F\xb0\xfc\xb7vjw\x9c,\x8eaI\xbb\x9d\xf8\xf1\
\xd6\xdf\x17\xedx\x22Y\x99\xadr\x0e9\xc5\x0bH\x0a\
y\xfe\xa6\xcd\xe9\x1aa\x1a\x1d\xd8}8\x92\x09\xe8v\
\xd2\x08,\xd5V\xfb\xb8\x87Gbu3\xdeQ\xcc\x98\
;7\xde\xd3a\xfb\xb2\x96\xf9S\x05\xc2\x06\xd2\xbf\x8f\
\x98\xcb\xfd3\xfd\xa9\xe7!\x0d\xd7'X\xc1\x0b\xc8\xd8\
`S\xce\xb3\x12\x92\xc8\x97\x9ag\xa8\xc44U\x90\xf9\
J\xe0w\x04\x14\x98\xc4\xf1\xa9BFSf\xeb\x86\xb7\
M\x80\x91\xa8\x09\xcf\x99\xbd\xe0\x8e\xe1\x1b\xfe\xb5 \xd5\
.3#\xd7\x93\xaeO\xb6X\x14\xbc\xec\x96\xc0fT\
\xec\xc1\x5c\xdb\xd4\x5c\xcd\xd6\xc0\xf5P\xc7\xb3\xa3-8\
\xba\xe9#F#\xf2\xd1l\xcd\xd3\x88F*\x87\x1a\x22\
\x82\xa3\xc3xW\x83N@\x80\x85Y\x0f\xe2B#\xff\
\xc1|\x0dTX,\xae\xff\x0b\xde\x88m \x04}\xee\
\xee&\xe1\xa5\x1f\xfa\xa5\xdf\xf9bC1\x01D\xc8A\
g\xffz\xf7\xe3\xddM\x10\xcc\xfe\x9d\xe5\xf7\x9dj\xd8\
\xe2\xcf\xc1\x00\xb7\x13\xe0\x0b\x83\x19\xda\xd0/\xef\xa2\x04\
\xb6\x10\xccu_C\xfe\x09\x83\xb4\x0d\xc8\x83\xabP'\
\xa2\x12\x02\xf5\x81\xc8\xda\xa5ij\x18$\x11v\x9a~\
,a\x0a\x1fP\xb6@\x00d\x89\xd47\xcb\x85j?\
,\x05\x0c8BT\xc2\xe4>\xfa\x89\xf2\x9eC\xadY\
\x0a\x0d*\xe2\xcd\xb4\xe3\x9c\xf6\x05\xc0}\xa5\x02J\x06\
\x9e8\x0a\xa0\xda:\xa1h\xa3\x82\x90[\xdf\x17\x1b\x91\
mT\x90\xbd\xf1\x97\xc2N}:\xdc\x03\xaa\x0d{\x8e\
\x19Q!S^t\x1cj\xda0\xc3\xedz3\x8f\xa3\
b\xc5\xf3Gv\xed\xf1\xc3\x13\xa4-i\x19-\x22\x9e\
7jp\xa8\xdb\xa0R\xac\x1fc?]n\x00\xe3\x06\
\xe2\x0c\x0a\xc9\xee\x11v\xfe \x8f\xd6=~\xa8Y\xcb\
<\x9ao\x8e\xd8A\xa2\xcf\xa0\xc7\xb4\xf6\xaa\xff;\xf7\
\xea\x14\x9b\xb6\x110m\xe2\xe2\xeefy\xd5\x8f\xf2a\
Q}\x98\xe9g\x9bu\x02\x89^] \x8c\xdbc\x7f\
\xce\xa1\x96\xfa\x00\xc0\xf4\x0a)\xc1\xda\xac\x04e\xb7D\
\x8a\xdbX\xac\x92\xc4Va\xcby5i\xd59P(\
\xcd\xd2n\xbd\x1d\x0f\xf8\x06J5Q\xb4\x0fw\x85~\
\x0dsj\x16w7%T\xfe\x1d\xd8\x98\xe4y\xbac\
Z\x848\xbd\x05\xec\xd8A@\xdd\x8a\x1b\x87n\x18\xb6\
\xcb\xec\xc9\xc1\xea\xb9\x00W\xd1\xc4=\xcc\x04l\x1c_\
\x0b\xca\x83\x9fG~Z\x0eh[Q`\x0cHP\xb8\
\xf22X544M]\xe6\xcc\x8c\xbfV,\x0b?\
\x89\xe2\xfd\xec\xbbw\xfc7\xff\xe7\x8d\xf2\x11\xa0\xfd\xee\
Zk\xe6\xacUb\xd6<\x80\x80\x09D\x94\x8cXG\
\xa7)\xd5\x22[\xa5\x85\xe2\xa1\xb7\xceWe\xb4\xb0\xc8\
\x89S\x17\xb9\xa50d\x00\xeb\xd1\xb6\xd0P\xc1\x10\x00\
p\xdaw\xfb?\x02\xbf\x8a\x06\xc5\xcc\x8c\xb8\xeb\xdd\x00\
N\x02Y\xd6\x93\x00:\xe9O\xe2\x88C\x1du\x9a\xde\
\x99Z\x9e\xe1\xf4{\xc1\xd8\xa1&\xb0b\x1e\x83\x5c\xcb\
_\xaf\x8b\x9b\xa9 @p#\x96O\xe6\xd3\x94\xea\xa6\
\xeb\xf6\xf2\xc1\xdfa\x93y\x16\x87\xcf\xe4\xd1\xca\x1b\x18\
\xeb\xbbk\x04A\xf3\xe3h\x99\xce\x8a\xd2\xcf\xcb\xebm\
\x1e\x95\x90\xb4k\x18\xff\xb38\xd7\xcay\xcd\x93\x06+\
Hm*\xa6g\x0a\x06<&\xd50\x8d\x1e\x85CG\
\x7f\xca\x808\x0a\xfe3\x84Ce\x8d\xc7\xc7\x84\xc4\xe9\
\xbe,&\x1c\x88\x89$\x1c\x87D\x0e\xc9\xcb\x81\x1b\x8f\
\x8c\x8di\xc2\x22\xce\xb6\xb3\xba\xb2\xba\x16\xdfQ\x8c\xe6\
nH\x95Cp\xfc\x0b\xc6\x0e!\x1e\xf3\x8d\xb0K\xfa\
\x99\xe7\xd9\xc0E\xaa\xfb\xea\x14kF\xaf\x13?\xbf\xe7\
\xf91\xef\xf1\x83`\x93l\x06\xfbZ5K\x9c\x07\xb1\
w\xa4\xbff4\x07c}Z{\x14f\x0f\x83\xda\x1c\
\x10\xd0\x12\xa6q\xc2O\xab\xa10\x97xZ\x00C\x13\
?_\x0b@j\xee\xa8y\x08\xe0\x80\xd6\xbeT1\xcf\
\x01h\x9c\x02\xb0\x1a\xea\xe9\x01\xfc\xba\x1e\xc8\xe8\x8e\xd1\
C\x00\x07\xb4\xf6\xb4\x94\x9e\x01\xd0\xa2'\xf0\xabF:\
\xc4\xef\x00g\xba\xa3tl\x182\x1e\xd9\x91\x19X2\
\x91\x01\xad\xf5\x9a\xd1\x10\xdf\x92\x09\xcf\x03h\xba;\xd3\
\x1d\x02H\xa91\x04\x90\xba\xae\x04\xc0A\xb7\xe6\xd8\xdc\
\x95\x008\x1e\xe2\xdb\x06\xf0\xdb\x8e`j\xd9;jI\
v\x91!q\xfcj\xb9\xdb\xa7GA<\xd8\xb0\x0f\x82\
\xb3\x1e\xec\xcf\x06\xa2m\xeel\xc9>b\xcb\xf6\x11\xdb\
<\x15Kg7\xe2j\xa8?\x1b\x80\x9e\xbd\xf3$>\
\xe8\xc9\x5c\xd0\xb3\xcf\x00x\xd2\x05\xab\x91\xfe\xa4Q\xac\
9\xcf\x1f\xc7\xd3e\xef\xc4\xe6\xdc1Q\xefT\xc6<\
z\x94\xf3Q\xbc\x03\x9e\x5c\x9d\xb2L\xd7\xdaT\x04Q\
Z\xe0oJ\xf0\xd7\x03\xf5{\x8e\xb1\x89s\xcc1\x8c\
\xe14Mg\x08\x03\x93\xa6~\x83\xfd\xafM\x0d\x0c\xd9\
\xd6F\x88y\xb8\xb3\xd5g\xe6s\x8e\x9f\xeb\x0b|\xac\
\xf1(C''K\xd1/\xf07\xe3\xcb\xf6\xbcK\xcd\
LO\x9c\xd85c\x9d1\xf5\xdd\x0d\xbe8~zP\
\x05\x90\x9b<~\xf9\xe2\xf0\xe5\xd1\xab\xe7\x8ed\xc0\xeb\
\x1f\x8ak\xaa\xa6\xad[\xca[\xc5\x82/\xdbu\x98U\
\x11\x98\xa9[\xaa\xedV4Z=z\xb6\xf2\xa0\x10j\
(\x81b\xa8\xd4\xd1MF\xa8\xa3@\xd9l\xd4\x1d=\
\xe8\x88\x97\xeanU\xf36\x9c\xaaQ5jM\x8f\xfa\
Q0\xff\xac\x80\xf4\xb7\x0a\xb5\xd8`\x5c\xcaH\xddY\
hE\x8dZ\xdf_F\x91\x80\xf62i\xfd*\xe9R\
\xcf\xb1\x8f/\x10\xfbd\x9e\xc5\xc5Y\xb7Y\x0e\xb5\xfa\
;J\xfd\x95h\xfd\xdc\x1d\x0f\xc0\xb5\xea\x97`\xa2Y\
\xe9\x97\x0a\xbd\x83\xea\xc4/\xf3h\xf7\x92\xe8\x8cRf\
P\xd5\x80O\xfb`y\xbaG\x09\xb8\x8d\xea\x10\xdd\xf5\
L\xcf\xf1^\x1dx\xad\xa2\x04\xb1_\x14\xe0\xb7\xf8\x0a\
kpj!lO\x5c\xdd\x04\xac\x99n:\xca\x0f\xe2\
I\xc5\x8bb\x88\x0f\xd5\x19QMK\xa7Lq\x98N\
\x98*\xae\xfdF\x97\xe8\x0evc\xae*\xaeu[O\
\xec\xc1I\x09*X\xbd\x81\xfb2U\xa9\x87\x1aX\x8e\
\x8e\x9aZ\xba\xe3\xa8x\xc1\xc1\xe0\x1f\xf8\xa6WO\xa3\
\xdf$\xf4l\xfbIU\xa1_\xacJ\xa2\x10\xc3\xd5\x1d\
Z\x8f\xe7\xcb\xe6o\xebD )E\xceP5fc\
\xd71\xdeZ\xd7\xcf\x94*\xcb.P\xd6s+\xd0\xfc\
\x03\xd0\xe0\xd6\xae\x9e\x7f8@\xad\xee%\xd5\xc2\xbc@\
\x0b\xc4\xcaV\x05p\x0dd\x9e\x80\xcck\xe7\x8eh\xa9\
\x88\x80\x0c3\x01\x98jH\xb0\x16\x1dMU`\xc7\xa4\
\xfaZ\x17\xea\xeb\xa8\x1e:\xf9I\xe4\xc6\xb8i\xa2\x09\
\xafr\xec\xec\x0b<\xbf\xd2\x05\xfd\xc4\xac\x82\x94\x89'\
\xd6\x8c\x89A\xa8\xd6\x9a\x1e\x22\xa7\xf5\xa1\x1bb^!\
w\x1c8\xe7b\xe0(C\xf5\x8e\x03g\xf7\xee\xc6\xc0\
\xd9R]\xdcK\x5c\xdf\x02\xc8Tq\xad|n\x14m\
\xb0\xcdPS\xa7\x1e\xa0\x06\x905\xeb\x05\xa8\xe0\xc2\x06\
\x05_\x9e\xccO\xb1\x07x\xa0nI\xd5\xf4.P\xd3\
e\xba\xab\xe2E\x02\x98\xab\x13[\x15\xb8\xb9Hn\x9d\
\xbe\xa2\xcb\x17W\xe3\x02\x1f\x83=\xd6\xf2Tq\xad}\
\xcc\x16\xd3n\xcc\xc3\x98nx\xaa\x83N\xe6P\xdd\x03\
\xbc\xf0Z7\x22Y\x15\x1c\xb2\x9e\x9dd\xb9\xba\xe4\x02\
\xc8\x98\xa5\x1b\x8e\x8a\xd7\xa7\x02\x8d^\x00\x1az\x96\x98\
\x19\xab1\x1b\x85\x17\xf1t\x1b\xa6\xce\x102\xd7U\xc5\
\xb5Y_\x99\x8epRK\xd2\xaf\x13+W\xf5\xa2]\
\x80@\xb6\xa0\x9a\xb0.\xd4a\xd9\x06\x1f\xa9\x91\xb1T\
\xe3 (+\xb2\x5c\x0b\xf3B/\xb31 \x99#E\
\xcc\xc1\xd0\x02h\x5cg\x10\x92\xe8Z\xe0T\x08\xa7-\
\xd9\x01zR\xe5\xaaZ\x17y\x18\x86$\xa8p\xca\xbd\
<U\xac\x12\x8fp\xaf\xael\xbd43\xb5\x8ef\xa6\
\xef\xa1`\x8c\xc5\xaf[\x9e\xa2|\xfdc\xca\x9e^-\
\xc9\xae\xdb_R\xe1\x8f\xa9\x9e\xa5\x98\x94\x94<NU\
\xf0\xb8\xccv\x0c\x86\x8fL\x14\x1d\xd53>T\xa5\x0e\
pa\x83\xd74\xf4\xba\xb8B\x8e\xdb/r\x88\xcd\x98\
\xa8qTJ\x9a\x1e\xf0`z\xbd\xe2\xa67\x0a\x966\
\xa2\x0b\x0c/\xea\x1a\xe7\xa0\xaa\x11/< \xbd\x97\x1d\
\x8c\x04Y\x9aBs\x96k\xc1&\x7f\xf0\xcbM\xce\xf1\
\x87v\x87\xce\xdefn\x86k\x0d\xb1H\xc4\xb8\x14\xe2\
j\xa4\xbfVO@\xdc\x83\xfe\x0f\xb0_6E\x1fr\
)\x0d\x03NN\xab\xe6\xf8\x1e\x00\xe9P\x05\xa9\x16b\
Gq\xd5\xc1\x1exO*P\xfb\x82zr*1\x9f\
\xff\xd0\xb3\x0c\xf3k\x1cf\xfc.\xe3\x9d\xe9!\xab\x82\
]C\xe2\xf2h\x8f\xbe\xcf\x1f\xb5\x8f&5\x10\xba|\
c\x22\xa9\xcb\x1f\xba\xccY\x8f\xf9\xfc,\xc7V_g\
\xa5\x19/\xd3\xade~\xcf\xf9\x04;~\xb2%\xfe\xdf\
\xcf\xb7\xb0\x03|\xe5\x08\xec\x0e\xbd\xaah`x\xc7L\
\x5c\x96\x19~\x8f\x8f\xb8\xc4\xf1\x16S-\xd8\xf1\x0d\xbc\
\xf6|\x1f\xcb* \xe2i\x16\x137\xd0\xdc.\xf4\xb5\
D\x5c\xe3\xf1\xe8\xaa>\xb8R~Q\x12\x90K\x1e\xb7\
k\x0c\x14y\xdc\xb2{<\xb0k\xcd\x06\xfb\xdc`\x07\
t\x8f\xee@\xa6!\x7f?tb\x11kr\x1d\xfc\x9d\
\xe9\xdd\xd5\xff\x00\x92*\xbe\xc7\
"

qt_resource_name = b"\
\x00\x0d\
\x01a\xd0\x87\
\x00m\
\x00a\x00i\x00n\x00_\x00i\x00c\x00o\x00n\x00.\x00s\x00v\x00g\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x01\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x8d\xcbE['\
"


def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)


def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)


qInitResources()
