default_assignments_module_names = ["assessments", "assessment"]
app_name = "QCanvas"
package_name = "qcanvas"
# Passed back to the launcher script to indicate that the program should be restarted (after an update)
updated_and_needs_restart_return_code = 144
